"use strict";
var tinymce = tinymce || {};

/*
 * TinyMCE plugin for inserting media from Frontify.
 *
 * Opens a child window and then uses the postMessage API to
 * receive the image to insert into the editor.
 *
 * Settings: (Settings can be done from TinyMceModule config in Episerver solution)
 *   frontifyUrl (required) - the base URL to Frontify (e.g. https://my.frontify.com)
 *   useIframeProxy (optional) - if set to true a window with iframe will be used for compability with IE11
 *   debugLog (optional) - if set to true logs debug messages
 */

tinymce.PluginManager.add('frontify', function (editor, url) {

    // polyfill Promise if needed
    if (typeof Promise !== 'function') {
        tinymce.ScriptLoader.loadScripts([url + '/lib/es6-promise.min.js'],
            function () {
                window.Promise = ES6Promise;
            });
    }

    var hostcontent = getHashParameterByName("context");
    hostcontent = (hostcontent !== null) ? hostcontent.replace("epi.cms.contentdata:///", "") : "";

    // reference to the window
    var fwWindow;
    var windowUrl;

    // url to iframe proxy page
    const iframeProxyPage = url + '/frontify.html';

    // url to error page
    const errorPage = url + '/error.html';

    // url to Frontify button icon
    const frontifyIcon = url + '/icon-48x48.png';

    // url to edit button icon
    const cropIcon = url + '/crop.svg';

    // true if Frontify window is open and communication is in progress,
    // false otherwise
    var fwWindowCommunicating = false;

    // use event when window is closing
    var eventDispatcher = new tinymce.util.EventDispatcher();

    // reference to the ping timeout so we can clear it
    // when window closes
    var pingTimer;

    // polyfill for location.origin
    if (typeof location.origin === 'undefined') {
        location.origin = location.protocol + '//' + location.host;
    }

    // origin of frontify / iframe proxy
    var targetOrigin;

    const logEnabled = editor.settings.debugLog;

    // features of the Frontify window
    const fwWindowFeatures = logEnabled ? '': 'width=1200,height=800,resizable=yes,scrollbars';
    const allowedtypes = editor.settings.frontifyallowedfiletypes;
    /*
     * Adds the button to insert Frontify media and handles
     * the insert action
     */
    editor.addButton('frontify-insert-media',
    {
        title: 'Insert',
        image: frontifyIcon,
        tooltip: 'Insert Frontify media',
        onclick: function () {
            verifyConfig()
                .then(function() {
                    // Focus tiny (to trigger save and publish events)
                    editor.focus();

                    // Open Frontify
                    openFrontifyWindow('insert');
                })
                .then(waitForWindowToClose)
                .then(function (media) { addMediaToEditor(media); })
                .catch(function (error) { handleError(error); });
        }
    });

    /*
     * Adds the button that edits Frontify media and handles
     * the edit action 
     */
    //editor.addButton('frontify-edit-media',
    //    {
    //        title: 'Edit',
    //        image: cropIcon,
    //        tooltip: 'Edit selected Frontify media',
    //        onclick: function () {
    //            verifyConfig()
    //                .then(function () { openFrontifyWindow('edit'); })
    //                .then(waitForWindowToClose)
    //                .then(function (media) { addMediaToEditor(media); })
    //                .catch(function (error) { handleError(error); });
    //        },
    //        onpostrender: monitorNodeChange
    //    });

    /*
     * Edit button is only enabled when we have
     * an img selected.
     */
    //function monitorNodeChange() {
    //    var btn = this;
    //    editor.on('NodeChange',
    //        function (e) {
    //            btn.disabled(!e.element || !e.element.nodeName || e.element.nodeName.toLowerCase() !== "img");
    //        });
    //}

    /*
     * Verifies all required editor configuration exist
     */
    function verifyConfig() {
        if (!editor.settings.frontifyUrl) {
            return Promise.reject(Error('Missing required editor configuration: frontifyUrl'));
        }

        if (editor.settings.useIframeProxy) {
            targetOrigin = location.origin;
        } else {
            targetOrigin = editor.settings.frontifyUrl;
        }

        // Make sure target origin does not have trailing slash
        if (targetOrigin.slice(-1) === '/') {
            targetOrigin = targetOrigin.slice(0, -1);
        }

        return Promise.resolve();
    }

    /*
     * Opens Frontify window in 'insert' or 'edit' mode
     */
    function openFrontifyWindow(mode) {
        try {
            windowUrl = createUrl(mode);
            //const windowName = 'FwTinyMce_' + editor.id;

            //logEnabled && console.info('Opening window with name: ' + windowName);

            //fwWindow = window.open(windowUrl, windowName, fwWindowFeatures);

            require(["epi-cms/legacy/LegacyDialogPopup"], function (LegacyDialogPopup) {
                fwWindow = new LegacyDialogPopup({
                    url: windowUrl,
                    onCallback: null,
                    dialogArguments: {},
                    dialogTitle: "Frontify Asset Chooser (" + allowedtypes + ")",
                    autoFit: false,
                    showCancelButton: true,
                    showLoadingOverlay: false,
                    features: { width: 1200, height: 900 }
                });
                fwWindow.actionContainerNode.hidden = true;
                fwWindow.show();
                //editor.fire("OpenWindow", {
                //    win: null
                //});
            });

            if (window.addEventListener) {
                window.addEventListener('message', messageEvent, false);
            } else if (window.attachEvent) {
                window.attachEvent('onmessage', messageEvent);
            }

            return Promise.resolve();
        } catch (e) {
            return Promise.reject(Error('Error opening Frontify window: ' + e.message));
        }
    }

    /*
     * Waits for event when external window has closed
     */
    function waitForWindowToClose() {
        return new Promise(function (resolve) {
            eventDispatcher.on('fwWinClosed', function (e) {
                eventDispatcher.off('fwWinClosed');
                resolve(e.media);
            });
        });
    }


    /*
     * Handle errors nicely
     */
    function handleError(error) {
        editor.windowManager.open({
            title: 'Frontify plugin error',
            url: errorPage,
            width: 700,
            height: 400
        },
            {
                errorMessage: error
            });
    }

    /*
     * Create absolute URL with required parameters to insert or edit media
     */
    function createUrl(mode) {
        // common params for all requests
        var params = {
            frontifyUrl: editor.settings.frontifyUrl,
            version: editor.settings.version
        };

        if (mode === 'edit') {
            // include mediaUrl for edit
            const selectedImage = tinymce.activeEditor.selection.getNode();

            // src for poster image for video (to edit video)
            const videoSrc = selectedImage.getAttribute("data-mce-p-poster");

            var mediaUrl = "";

            if (videoSrc) {
                mediaUrl = videoSrc;
            } else {
                mediaUrl = selectedImage.getAttribute("src");
            }

            params.mediaUrl = mediaUrl;
        }

        const query = Object.keys(params)
            .map(function (k) { return encodeURIComponent(k) + '=' + encodeURIComponent(params[k]); })
            .join('&');

        var baseUrl;

        // check if we should call Frontify directly
        // or use iframe proxy
        if (editor.settings.useIframeProxy) {
            baseUrl = iframeProxyPage;
        } else {
            baseUrl = editor.settings.frontifyUrl;
        }

        logEnabled && console.info('Frontify URL: ' + baseUrl + '?' + query);
        return baseUrl + '?' + query;
    }
   
    /*
     * Handler for incoming postMessages
     */
    function messageEvent(event) {
        // Ignore messages not for us.
        if (event.origin !== targetOrigin) {
            logEnabled && console.info('Message received to another origin: (' + event.origin + ' != ' + targetOrigin + '): ' + event.data);
            return;
        }

        logEnabled && console.log(event.data, event);

        //take care of tokenRequested when iframe is loaded
        if (event.data.configurationRequested) {
            var fileTypes = editor.settings.frontifyallowedfiletypes.replace(",", "").split('.');
            fwWindow.wrapper.containerIframe.contentWindow.postMessage({ "token": editor.settings.apitoken, multiSelectionAllowed: false, mode: editor.settings.choosermode, fileTypes: fileTypes }, location.origin);
            return;
        }
        else if (event.data.aborted) {
            logEnabled && console.log("cancel, closing window");
            closeWindow();
        }
        else if (event.data.assetsChosen) {
            if (event.data.assetsChosen.length > 0) {
                eventDispatcher.fire('fwWinClosed', { media: event.data.assetsChosen[0] }); 
            }
        }
    }

    /*
     * Inserts media (if any) into the editor
     */
    function addMediaToEditor(media) {
        if (media) {

            ajax(editor.settings.frontifymodulepage + "authentication/fetch?hostcontent="
                + hostcontent
                + "&imageid=" + media.id
                + "&ext=" + media.ext
                + "&projectid=" + media.project
                + "&allowedfiletypes=" + editor.settings.frontifyallowedfiletypes,
                function (xhttp) {
                    //debugger 
                    //success
                    logEnabled && console.log("image found", xhttp.responseText);

                    editor.insertContent(xhttp.responseText);
                    closeWindow(); 
                },
                function (xhttp) {
                    closeWindow(); 
                    handleError(xhttp.statusText);
                });
        }
    }

    /*
     * Closes and cleans up the window and handlers
     */
    function closeWindow() {
        try {
            //editor.fire("CloseWindow", {
            //    win: null
            //});
            window.removeEventListener('message', messageEvent);
            fwWindow.hide();
        } catch (e) {

        } finally {
            fwWindowCommunicating = false;
            eventDispatcher.fire('fwWinClosed', { media: null });
        }
    }

    function ajax(url, successFunc, errFunc) {
        var xhttp;
        xhttp = new XMLHttpRequest();
        xhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                successFunc(this); 
            }
            if (this.readyState == 4 && this.status != 200) {
                errFunc(this);
            }
        };
        xhttp.open("GET", url, true);
        xhttp.send();
    }

    function getHashParameterByName(name) {
        const url = window.location.hash;
        name = name.replace(/[\[\]]/g, '\\$&');
        const regex = new RegExp('[?#&]' + name + '(=([^&#]*)|&|#|$)');
        const results = regex.exec(url);

        if (!results) {
            return null;
        }

        if (!results[2]) {
            return '';
        }

        return decodeURIComponent(results[2].replace(/\+/g, ' '));
    }

    /*
     * TinyMCE metadata shows up in the ? menu
     */
    return {
        getMetadata: function () {
            return {
                name: 'Frontify',
                url: 'https://www.frontify.com'
            };
        }
    };
});